@@ -12,20 +12,15 @@
 use std::os::fd::{AsFd, AsRawFd};
 
 use libseccomp::ScmpNotifResp;
-use nix::{
-    errno::Errno,
-    fcntl::AtFlags,
-    unistd::{faccessat, AccessFlags},
-    NixPath,
-};
+use nix::{errno::Errno, fcntl::AtFlags, unistd::AccessFlags, NixPath};
 
 use crate::{
-    compat::{fstatx, STATX_MODE},
+    compat::{fstatx, AT_EACCESS, STATX_MODE},
     config::PROC_FILE,
-    cookie::safe_linkat,
+    cookie::{safe_faccess, safe_linkat},
     kernel::{syscall_path_handler, to_atflags},
     lookup::{FileType, FsFlags},
-    path::{XPath, XPathBuf},
+    path::XPathBuf,
     req::{PathArgs, SysArg, SysFlags, UNotifyEventRequest},
 };
 
@@ -117,11 +112,7 @@ fn syscall_link_handler(
     // Note, linkat does not follow symbolic links in old path by
     // default unless AT_SYMLINK_FOLLOW flag is passed. As such,
     // AT_SYMLINK_NOFOLLOW is an invalid flag for linkat.
-    let fd = old_path
-        .dir
-        .as_ref()
-        .map(|fd| fd.as_fd())
-        .ok_or(Errno::EBADF)?;
+    let fd = old_path.dir.as_ref().ok_or(Errno::EBADF)?;
     assert!(old_path.base().is_empty()); // MUST_PATH!
 
     // SAFETY: Restrictions a la CONFIG_GRKERNSEC_LINK.
@@ -132,11 +123,7 @@ fn syscall_link_handler(
     safe_linkat(
         PROC_FILE(),
         &XPathBuf::from_self_fd(fd.as_raw_fd())?,
-        new_path
-            .dir
-            .as_ref()
-            .map(|fd| fd.as_fd())
-            .ok_or(Errno::EBADF)?,
+        new_path.dir.as_ref().ok_or(Errno::EBADF)?,
         new_path.base(),
         AtFlags::AT_SYMLINK_FOLLOW,
     )
@@ -173,11 +160,10 @@ fn safe_hardlink_source<Fd: AsFd>(fd: Fd, typ: FileType) -> Result<(), Errno> {
     }
 
     // Caller must have both read and write access to the file.
-    faccessat(
-        &fd,
-        XPath::empty(),
+    safe_faccess(
+        fd,
         AccessFlags::R_OK | AccessFlags::W_OK,
-        crate::compat::AT_EACCESS | AtFlags::AT_EMPTY_PATH,
+        AT_EACCESS | AtFlags::AT_EMPTY_PATH,
     )
     .or(Err(Errno::EPERM))
 }
