@@ -2,32 +2,39 @@
 // Syd: rock-solid application kernel
 // src/kernel/ptrace/exec.rs: ptrace exec handlers
 //
-// Copyright (c) 2023, 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2023, 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 //
 // SPDX-License-Identifier: GPL-3.0
 
 use std::{fs::File, io::Seek, os::fd::AsRawFd};
 
+use data_encoding::HEXLOWER;
 use nix::{
     errno::Errno,
-    fcntl::{AtFlags, OFlag, ResolveFlag},
+    fcntl::{AtFlags, OFlag},
     sys::signal::{kill, Signal},
     unistd::Pid,
     NixPath,
 };
 
 use crate::{
+    compat::ResolveFlag,
     config::{PROC_FILE, PTRACE_DATA_EXECVE, PTRACE_DATA_EXECVEAT},
+    confine::{is_valid_ptr, scmp_arch, SydArch},
+    debug,
     elf::{ElfError, ElfFileType, ElfType, ExecutableFile, LinkingType},
     err::err2no,
     error,
-    fs::{is_executable, AT_EXECVE_CHECK},
+    fd::{is_executable, AT_EXECVE_CHECK},
     kernel::{sandbox_path, to_atflags},
+    log_enabled,
     lookup::{safe_open_msym, FileType, FsFlags, MaybeFd},
     path::XPathBuf,
+    proc::proc_get_vma,
     ptrace::ptrace_syscall_info,
     req::{RemoteProcess, SysArg, SysFlags},
     sandbox::{Action, Capability, IntegrityError, SandboxGuard},
+    syslog::LogLevel,
     warn,
 };
 
@@ -91,8 +98,12 @@ pub(crate) fn sysenter_exec(
     // Read remote path.
     let process = RemoteProcess::new(pid);
 
+    #[expect(clippy::disallowed_methods)]
+    let arch: SydArch = scmp_arch(info.arch).unwrap().into();
+
     // SAFETY: This is a ptrace hook, the PID cannot be validated.
-    let (mut path, _, _, empty_path) = process.read_path(sandbox, data.args, arg, None)?;
+    let (mut path, _, _, empty_path) =
+        process.read_path(sandbox, arch.into(), data.args, arg, None)?;
 
     // Call sandbox access checker if Exec sandboxing is on.
     let caps = sandbox.getcaps(Capability::CAP_EXEC | Capability::CAP_TPE);
@@ -151,7 +162,7 @@ pub(crate) fn sysenter_exec(
     if let Some(action) = sandbox.check_segvguard(path.abs()) {
         if action != Action::Filter {
             let (_, bin) = path.abs().split();
-            error!("ctx": "segvguard",
+            error!("ctx": "exec", "op": "segvguard",
                 "msg": format!("max crashes {} exceeded, execution of `{bin}' denied",
                     sandbox.segvguard_maxcrashes),
                 "tip": "increase `segvguard/maxcrashes'",
@@ -188,7 +199,7 @@ pub(crate) fn sysenter_exec(
         if !matches!(action, Action::Allow | Action::Filter) {
             // TODO: Fix proc_mmap to work in ptrace hooks.
             let msg = msg.as_deref().unwrap_or("?");
-            error!("ctx": "trusted_path_execution",
+            error!("ctx": "exec", "op": "trusted_path_execution",
                 "msg": format!("exec from untrusted path blocked: {msg}"),
                 "pid": pid.as_raw(), "path": path.abs(),
                 "sys": syscall_name, "arch": info.arch, "args": data.args,
@@ -219,7 +230,7 @@ pub(crate) fn sysenter_exec(
     // We prefer this over WANT_READ to ensure
     // we do not share OFD with the sandbox process.
     assert!(
-        path.base.is_empty(),
+        path.base().is_empty(),
         "BUG: MUST_PATH returned a directory for exec, report a bug!"
     );
     let mut file = match path.dir.take() {
@@ -255,7 +266,7 @@ pub(crate) fn sysenter_exec(
         }
         Err(ElfError::BadMagic) => {
             if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
-                error!("ctx": "parse_elf",
+                error!("ctx": "exec", "op": "parse_elf",
                     "msg": "invalid ELF file",
                     "pid": pid.as_raw(), "path": path.abs());
             }
@@ -263,7 +274,7 @@ pub(crate) fn sysenter_exec(
         }
         Err(ElfError::Malformed) => {
             if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
-                error!("ctx": "parse_elf",
+                error!("ctx": "exec", "op": "parse_elf",
                     "msg": "malformed ELF file",
                     "pid": pid.as_raw(), "path": path.abs());
             }
@@ -274,7 +285,7 @@ pub(crate) fn sysenter_exec(
     let is_script = exe == ExecutableFile::Script;
     if is_script && deny_script {
         if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
-            error!("ctx": "deny_exec_script",
+            error!("ctx": "exec", "op": "deny_exec_script",
                 "msg": "script execution denied",
                 "pid": pid.as_raw(), "path": path.abs(),
                 "tip": "configure `trace/deny_exec_script:0'",
@@ -294,7 +305,7 @@ pub(crate) fn sysenter_exec(
         )
     {
         if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
-            error!("ctx": "check_elf",
+            error!("ctx": "exec", "op": "check_elf",
                 "msg": "ld.so(8) exec-indirection prevented",
                 "pid": pid.as_raw(), "path": path.abs(),
                 "tip": "configure `trace/allow_unsafe_exec_ldso:1'",
@@ -315,7 +326,8 @@ pub(crate) fn sysenter_exec(
 
     if !is_script && restrict_xs && matches!(exe, ExecutableFile::Elf { xs: true, .. }) {
         if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
-            error!("ctx": "check_elf", "msg": "ELF has executable stack",
+            error!("ctx": "exec", "op": "check_elf",
+                "msg": "ELF has executable stack",
                 "pid": pid.as_raw(), "path": path.abs(),
                 "tip": "configure `trace/allow_unsafe_exec_stack:1'",
                 "exe": format!("{exe}"));
@@ -334,7 +346,8 @@ pub(crate) fn sysenter_exec(
         )
     {
         if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
-            error!("ctx": "check_elf", "msg": "32-bit ELF execution prevented",
+            error!("ctx": "exec", "op": "check_elf",
+                "msg": "32-bit ELF execution prevented",
                 "pid": pid.as_raw(), "path": path.abs(),
                 "tip": "configure `trace/deny_exec_elf32:0'",
                 "exe": format!("{exe}"));
@@ -353,7 +366,8 @@ pub(crate) fn sysenter_exec(
         )
     {
         if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
-            error!("ctx": "check_elf", "msg": "dynamic linked ELF execution prevented",
+            error!("ctx": "exec", "op": "check_elf",
+                "msg": "dynamic linked ELF execution prevented",
                 "pid": pid.as_raw(), "path": path.abs(),
                 "tip": "configure `trace/deny_exec_elf_dynamic:0'",
                 "exe": format!("{exe}"));
@@ -372,7 +386,8 @@ pub(crate) fn sysenter_exec(
         )
     {
         if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
-            error!("ctx": "check_elf", "msg": "static linked ELF execution prevented",
+            error!("ctx": "exec", "op": "check_elf",
+                "msg": "static linked ELF execution prevented",
                 "pid": pid.as_raw(), "path": path,
                 "tip": "configure `trace/deny_exec_elf_static:0'",
                 "exe": format!("{exe}"));
@@ -388,13 +403,13 @@ pub(crate) fn sysenter_exec(
         match sandbox.check_force2(path.abs(), &mut file) {
             Ok(Action::Allow) => {}
             Ok(Action::Warn) => {
-                warn!("ctx": "verify_elf", "act": Action::Warn,
+                warn!("ctx": "exec", "op": "verify_elf", "act": Action::Warn,
                     "pid": pid.as_raw(), "path": path.abs(),
                     "tip": format!("configure `force+{path}:<checksum>'"));
             }
             Ok(Action::Filter) => return Err(Errno::EACCES),
             Ok(act @ (Action::Deny | Action::Panic)) => {
-                warn!("ctx": "verify_elf", "act": act,
+                warn!("ctx": "exec", "op": "verify_elf", "act": act,
                     "pid": pid.as_raw(), "path": path.abs(),
                     "tip": format!("configure `force+{path}:<checksum>'"));
                 return Err(Errno::EACCES);
@@ -402,34 +417,34 @@ pub(crate) fn sysenter_exec(
             //Do NOT panic the main thread!
             //Ok(Action::Panic) => panic!(),
             Ok(Action::Stop) => {
-                warn!("ctx": "verify_elf", "act": Action::Stop,
+                warn!("ctx": "exec", "op": "verify_elf", "act": Action::Stop,
                     "pid": pid.as_raw(), "path": path.abs(),
                     "tip": format!("configure `force+{path}:<checksum>'"));
                 let _ = kill(pid, Some(Signal::SIGSTOP));
                 return Err(Errno::EACCES);
             }
             Ok(Action::Abort) => {
-                warn!("ctx": "verify_elf", "act": Action::Abort,
+                warn!("ctx": "exec", "op": "verify_elf", "act": Action::Abort,
                     "pid": pid.as_raw(), "path": path.abs(),
                     "tip": format!("configure `force+{path}:<checksum>'"));
                 let _ = kill(pid, Some(Signal::SIGABRT));
                 return Err(Errno::EACCES);
             }
             Ok(Action::Kill) => {
-                warn!("ctx": "verify_elf", "act": Action::Kill,
+                warn!("ctx": "exec", "op": "verify_elf", "act": Action::Kill,
                     "pid": pid.as_raw(), "path": path.abs(),
                     "tip": format!("configure `force+{path}:<checksum>'"));
                 let _ = kill(pid, Some(Signal::SIGKILL));
                 return Err(Errno::EACCES);
             }
             Ok(Action::Exit) => {
-                error!("ctx": "verify_elf", "act": Action::Exit,
+                error!("ctx": "exec", "op": "verify_elf", "act": Action::Exit,
                     "pid": pid.as_raw(), "path": path.abs(),
                     "tip": format!("configure `force+{path}:<checksum>'"));
                 std::process::exit(libc::EACCES);
             }
             Err(IntegrityError::Sys(errno)) => {
-                error!("ctx": "verify_elf",
+                error!("ctx": "exec", "op": "verify_elf",
                     "msg": format!("system error during ELF checksum calculation: {errno}"),
                     "pid": pid.as_raw(), "path": path.abs(),
                     "tip": format!("configure `force+{path}:<checksum>'"));
@@ -441,7 +456,7 @@ pub(crate) fn sysenter_exec(
                 found,
             }) => {
                 if !matches!(action, Action::Allow | Action::Filter) {
-                    error!("ctx": "verify_elf", "act": action,
+                    error!("ctx": "exec", "op": "verify_elf", "act": action,
                         "msg": format!("ELF checksum mismatch: {found} is not {expected}"),
                         "pid": pid.as_raw(), "path": path.abs(),
                         "tip": format!("configure `force+{path}:<checksum>'"));
@@ -479,5 +494,61 @@ pub(crate) fn sysenter_exec(
         return Err(Errno::ECANCELED);
     }
 
+    if log_enabled!(LogLevel::Debug) {
+        let exe = exe.to_string();
+        let path = path.abs();
+        let ip_vma = proc_get_vma(pid, info.instruction_pointer).ok();
+        let sp_vma = proc_get_vma(pid, info.stack_pointer).ok();
+
+        let ip = info.instruction_pointer;
+        let sp = info.stack_pointer;
+
+        let process = RemoteProcess::new(pid);
+        let ip_mem = if is_valid_ptr(ip, arch.into()) {
+            let mut ip_mem = [0u8; 64];
+            // SAFETY: This is a ptrace hook, the PID cannot be validated.
+            match unsafe { process.read_mem(arch.into(), &mut ip_mem, ip, 64) } {
+                Ok(_) => Some(ip_mem),
+                Err(Errno::EFAULT) => None,
+                Err(Errno::ESRCH) => return Err(Errno::ESRCH),
+                Err(_) => {
+                    // SAFETY: Process is alive, but
+                    // we cannot read memory: Terminate!
+                    let _ = kill(pid, Some(Signal::SIGKILL));
+                    return Err(Errno::ESRCH);
+                }
+            }
+        } else {
+            None
+        };
+
+        let sp_mem = if is_valid_ptr(sp, arch.into()) {
+            let mut sp_mem = [0u8; 64];
+            // SAFETY: This is a ptrace hook, the PID cannot be validated.
+            match unsafe { process.read_mem(arch.into(), &mut sp_mem, sp, 64) } {
+                Ok(_) => Some(sp_mem),
+                Err(Errno::EFAULT) => None,
+                Err(Errno::ESRCH) => return Err(Errno::ESRCH),
+                Err(_) => {
+                    // SAFETY: Process is alive, but
+                    // we cannot read memory: Terminate!
+                    let _ = kill(process.pid, Some(Signal::SIGKILL));
+                    return Err(Errno::ESRCH);
+                }
+            }
+        } else {
+            None
+        };
+        let ip_mem = ip_mem.map(|ip_mem| HEXLOWER.encode(&ip_mem));
+        let sp_mem = sp_mem.map(|sp_mem| HEXLOWER.encode(&sp_mem));
+
+        debug!("ctx": "exec", "op": "verify_exec",
+            "msg": format!("execution of `{path}' of type {exe} resumed"),
+            "pid": pid.as_raw(), "path": &path, "exe": &exe, "args": data.args,
+            "ip": info.instruction_pointer, "sp": info.stack_pointer,
+            "ip_mem": ip_mem, "sp_mem": sp_mem,
+            "ip_vma": ip_vma, "sp_vma": sp_vma);
+    }
+
     Ok((file, exe))
 }
