@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/kernel/setid.rs: Set UID/GID syscall handlers
 //
-// Copyright (c) 2023, 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2023, 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 //
 // SPDX-License-Identifier: GPL-3.0
 
@@ -16,11 +16,11 @@ use crate::{
     caps,
     config::{GID_MIN, UID_MIN},
     confine::safe_drop_cap,
-    hook::UNotifyEventRequest,
+    req::UNotifyEventRequest,
     warn,
 };
 
-#[allow(clippy::cognitive_complexity)]
+#[expect(clippy::cognitive_complexity)]
 pub(crate) fn sys_setuid(request: UNotifyEventRequest) -> ScmpNotifResp {
     syscall_handler!(request, |request: UNotifyEventRequest| {
         let req = request.scmpreq;
@@ -31,7 +31,7 @@ pub(crate) fn sys_setuid(request: UNotifyEventRequest) -> ScmpNotifResp {
         if target_uid.as_raw() <= UID_MIN.as_raw() {
             // SAFETY: This is already asserted with the parent
             // seccomp-bpf filter, this is the second layer.
-            return Ok(request.fail_syscall(Errno::EACCES));
+            return Err(Errno::EACCES);
         } else if source_uid == target_uid {
             // SAFETY: No UID change or no ptr-deref in check.
             return unsafe { Ok(request.continue_syscall()) };
@@ -39,11 +39,11 @@ pub(crate) fn sys_setuid(request: UNotifyEventRequest) -> ScmpNotifResp {
 
         let sandbox = request.get_sandbox();
         let allowed = sandbox.chk_uid_transit(source_uid, target_uid);
-        let verbose = sandbox.verbose;
+        let log_scmp = sandbox.log_scmp();
         drop(sandbox); // release the read lock.
 
         if !allowed {
-            if verbose {
+            if log_scmp {
                 warn!("ctx": "safesetid", "err": libc::EACCES,
                     "sys": request.syscall, "req": request,
                     "target_uid": target_uid.as_raw(), "source_uid": source_uid.as_raw(),
@@ -63,7 +63,7 @@ pub(crate) fn sys_setuid(request: UNotifyEventRequest) -> ScmpNotifResp {
 
         // SAFETY: nix version of setuid does not allow -1 as argument.
         if let Err(errno) = Errno::result(unsafe { libc::setuid(target_uid.as_raw()) }) {
-            if verbose {
+            if log_scmp {
                 warn!("ctx": "safesetid", "err": errno as i32,
                     "sys": request.syscall, "req": request,
                     "target_uid": target_uid.as_raw(), "source_uid": source_uid.as_raw(),
@@ -93,7 +93,7 @@ pub(crate) fn sys_setuid(request: UNotifyEventRequest) -> ScmpNotifResp {
     })
 }
 
-#[allow(clippy::cognitive_complexity)]
+#[expect(clippy::cognitive_complexity)]
 pub(crate) fn sys_setgid(request: UNotifyEventRequest) -> ScmpNotifResp {
     syscall_handler!(request, |request: UNotifyEventRequest| {
         let req = request.scmpreq;
@@ -104,7 +104,7 @@ pub(crate) fn sys_setgid(request: UNotifyEventRequest) -> ScmpNotifResp {
         if target_gid.as_raw() <= GID_MIN.as_raw() {
             // SAFETY: This is already asserted with the parent
             // seccomp-bpf filter, this is the second layer.
-            return Ok(request.fail_syscall(Errno::EACCES));
+            return Err(Errno::EACCES);
         } else if source_gid == target_gid {
             // SAFETY: No GID change or no ptr-deref in check.
             return unsafe { Ok(request.continue_syscall()) };
@@ -112,11 +112,11 @@ pub(crate) fn sys_setgid(request: UNotifyEventRequest) -> ScmpNotifResp {
 
         let sandbox = request.get_sandbox();
         let allowed = sandbox.chk_gid_transit(source_gid, target_gid);
-        let verbose = sandbox.verbose;
+        let log_scmp = sandbox.log_scmp();
         drop(sandbox); // release the read lock.
 
         if !allowed {
-            if verbose {
+            if log_scmp {
                 warn!("ctx": "safesetid", "err": libc::EACCES,
                     "sys": request.syscall, "req": request,
                     "target_gid": target_gid.as_raw(), "source_gid": source_gid.as_raw(),
@@ -136,7 +136,7 @@ pub(crate) fn sys_setgid(request: UNotifyEventRequest) -> ScmpNotifResp {
 
         // SAFETY: nix version of setgid does not allow -1 as argument.
         if let Err(errno) = Errno::result(unsafe { libc::setgid(target_gid.as_raw()) }) {
-            if verbose {
+            if log_scmp {
                 warn!("ctx": "safesetid", "err": errno as i32,
                     "sys": request.syscall, "req": request,
                     "target_gid": target_gid.as_raw(), "source_gid": source_gid.as_raw(),
@@ -166,13 +166,12 @@ pub(crate) fn sys_setgid(request: UNotifyEventRequest) -> ScmpNotifResp {
     })
 }
 
-#[allow(clippy::cognitive_complexity)]
+#[expect(clippy::cognitive_complexity)]
 pub(crate) fn sys_setreuid(request: UNotifyEventRequest) -> ScmpNotifResp {
     syscall_handler!(request, |request: UNotifyEventRequest| {
         let req = request.scmpreq;
 
-        #[allow(clippy::cast_possible_truncation)]
-        #[allow(clippy::cast_possible_wrap)]
+        #[expect(clippy::cast_possible_truncation)]
         let target_ruid = match req.data.args[0] as i32 {
             -1 => None,
             n if n >= 0 => Some(Uid::from_raw(
@@ -180,8 +179,7 @@ pub(crate) fn sys_setreuid(request: UNotifyEventRequest) -> ScmpNotifResp {
             )),
             _ => return Err(Errno::EINVAL),
         };
-        #[allow(clippy::cast_possible_truncation)]
-        #[allow(clippy::cast_possible_wrap)]
+        #[expect(clippy::cast_possible_truncation)]
         let target_euid = match req.data.args[1] as i32 {
             -1 => None,
             n if n >= 0 => Some(Uid::from_raw(
@@ -205,7 +203,7 @@ pub(crate) fn sys_setreuid(request: UNotifyEventRequest) -> ScmpNotifResp {
             if target_ruid.as_raw() <= UID_MIN.as_raw() {
                 // SAFETY: This is already asserted with the parent
                 // seccomp-bpf filter, this is the second layer.
-                return Ok(request.fail_syscall(Errno::EACCES));
+                return Err(Errno::EACCES);
             } else if source_ruid != target_ruid {
                 change = true;
             }
@@ -214,7 +212,7 @@ pub(crate) fn sys_setreuid(request: UNotifyEventRequest) -> ScmpNotifResp {
             if target_euid.as_raw() <= UID_MIN.as_raw() {
                 // SAFETY: This is already asserted with the parent
                 // seccomp-bpf filter, this is the second layer.
-                return Ok(request.fail_syscall(Errno::EACCES));
+                return Err(Errno::EACCES);
             } else if source_euid != target_euid {
                 change = true;
             }
@@ -226,13 +224,13 @@ pub(crate) fn sys_setreuid(request: UNotifyEventRequest) -> ScmpNotifResp {
         }
 
         let sandbox = request.get_sandbox();
-        let verbose = sandbox.verbose;
+        let log_scmp = sandbox.log_scmp();
 
         // SAFETY: We do not support RUID != EUID
         if let Some(target_ruid) = target_ruid {
             if let Some(target_euid) = target_euid {
                 if target_ruid != target_euid {
-                    if verbose {
+                    if log_scmp {
                         warn!("ctx": "safesetid", "err": libc::EACCES,
                             "sys": request.syscall, "req": &request,
                             "target_euid": target_euid.as_raw(), "target_ruid": target_ruid.as_raw(),
@@ -272,7 +270,7 @@ pub(crate) fn sys_setreuid(request: UNotifyEventRequest) -> ScmpNotifResp {
         let target_ruid = target_ruid.map(|uid| i64::from(uid.as_raw())).unwrap_or(-1);
         let target_euid = target_euid.map(|uid| i64::from(uid.as_raw())).unwrap_or(-1);
         if !allowed {
-            if verbose {
+            if log_scmp {
                 warn!("ctx": "safesetid", "err": libc::EACCES,
                     "sys": request.syscall, "req": request,
                     "target_euid": target_euid, "target_ruid": target_ruid,
@@ -298,7 +296,7 @@ pub(crate) fn sys_setreuid(request: UNotifyEventRequest) -> ScmpNotifResp {
                 libc::syscall(libc::SYS_setreuid, target_ruid, target_euid)
             })
         {
-            if verbose {
+            if log_scmp {
                 warn!("ctx": "safesetid", "err": libc::EACCES,
                     "sys": request.syscall, "req": request,
                     "target_euid": target_euid, "target_ruid": target_ruid,
@@ -330,13 +328,12 @@ pub(crate) fn sys_setreuid(request: UNotifyEventRequest) -> ScmpNotifResp {
     })
 }
 
-#[allow(clippy::cognitive_complexity)]
+#[expect(clippy::cognitive_complexity)]
 pub(crate) fn sys_setregid(request: UNotifyEventRequest) -> ScmpNotifResp {
     syscall_handler!(request, |request: UNotifyEventRequest| {
         let req = request.scmpreq;
 
-        #[allow(clippy::cast_possible_truncation)]
-        #[allow(clippy::cast_possible_wrap)]
+        #[expect(clippy::cast_possible_truncation)]
         let target_rgid = match req.data.args[0] as i32 {
             -1 => None,
             n if n >= 0 => Some(Gid::from_raw(
@@ -344,8 +341,7 @@ pub(crate) fn sys_setregid(request: UNotifyEventRequest) -> ScmpNotifResp {
             )),
             _ => return Err(Errno::EINVAL),
         };
-        #[allow(clippy::cast_possible_truncation)]
-        #[allow(clippy::cast_possible_wrap)]
+        #[expect(clippy::cast_possible_truncation)]
         let target_egid = match req.data.args[1] as i32 {
             -1 => None,
             n if n >= 0 => Some(Gid::from_raw(
@@ -369,7 +365,7 @@ pub(crate) fn sys_setregid(request: UNotifyEventRequest) -> ScmpNotifResp {
             if target_rgid.as_raw() <= GID_MIN.as_raw() {
                 // SAFETY: This is already asserted with the parent
                 // seccomp-bpf filter, this is the second layer.
-                return Ok(request.fail_syscall(Errno::EACCES));
+                return Err(Errno::EACCES);
             } else if source_rgid != target_rgid {
                 change = true;
             }
@@ -378,7 +374,7 @@ pub(crate) fn sys_setregid(request: UNotifyEventRequest) -> ScmpNotifResp {
             if target_egid.as_raw() <= GID_MIN.as_raw() {
                 // SAFETY: This is already asserted with the parent
                 // seccomp-bpf filter, this is the second layer.
-                return Ok(request.fail_syscall(Errno::EACCES));
+                return Err(Errno::EACCES);
             } else if source_egid != target_egid {
                 change = true;
             }
@@ -390,13 +386,13 @@ pub(crate) fn sys_setregid(request: UNotifyEventRequest) -> ScmpNotifResp {
         }
 
         let sandbox = request.get_sandbox();
-        let verbose = sandbox.verbose;
+        let log_scmp = sandbox.log_scmp();
 
         // SAFETY: We do not support Rgid != Egid
         if let Some(target_rgid) = target_rgid {
             if let Some(target_egid) = target_egid {
                 if target_rgid != target_egid {
-                    if verbose {
+                    if log_scmp {
                         warn!("ctx": "safesetid", "err": libc::EACCES,
                             "sys": request.syscall, "req": &request,
                             "target_egid": target_egid.as_raw(), "target_rgid": target_rgid.as_raw(),
@@ -436,7 +432,7 @@ pub(crate) fn sys_setregid(request: UNotifyEventRequest) -> ScmpNotifResp {
         let target_rgid = target_rgid.map(|gid| i64::from(gid.as_raw())).unwrap_or(-1);
         let target_egid = target_egid.map(|gid| i64::from(gid.as_raw())).unwrap_or(-1);
         if !allowed {
-            if verbose {
+            if log_scmp {
                 warn!("ctx": "safesetid", "err": libc::EACCES,
                     "sys": request.syscall, "req": request,
                     "target_egid": target_egid, "target_rgid": target_rgid,
@@ -462,7 +458,7 @@ pub(crate) fn sys_setregid(request: UNotifyEventRequest) -> ScmpNotifResp {
                 libc::syscall(libc::SYS_setregid, target_rgid, target_egid)
             })
         {
-            if verbose {
+            if log_scmp {
                 warn!("ctx": "safesetid", "err": errno as i32,
                     "sys": request.syscall, "req": request,
                     "target_egid": target_egid, "target_rgid": target_rgid,
@@ -494,13 +490,12 @@ pub(crate) fn sys_setregid(request: UNotifyEventRequest) -> ScmpNotifResp {
     })
 }
 
-#[allow(clippy::cognitive_complexity)]
+#[expect(clippy::cognitive_complexity)]
 pub(crate) fn sys_setresuid(request: UNotifyEventRequest) -> ScmpNotifResp {
     syscall_handler!(request, |request: UNotifyEventRequest| {
         let req = request.scmpreq;
 
-        #[allow(clippy::cast_possible_truncation)]
-        #[allow(clippy::cast_possible_wrap)]
+        #[expect(clippy::cast_possible_truncation)]
         let target_ruid = match req.data.args[0] as i32 {
             -1 => None,
             n if n >= 0 => Some(Uid::from_raw(
@@ -508,8 +503,7 @@ pub(crate) fn sys_setresuid(request: UNotifyEventRequest) -> ScmpNotifResp {
             )),
             _ => return Err(Errno::EINVAL),
         };
-        #[allow(clippy::cast_possible_truncation)]
-        #[allow(clippy::cast_possible_wrap)]
+        #[expect(clippy::cast_possible_truncation)]
         let target_euid = match req.data.args[1] as i32 {
             -1 => None,
             n if n >= 0 => Some(Uid::from_raw(
@@ -517,8 +511,7 @@ pub(crate) fn sys_setresuid(request: UNotifyEventRequest) -> ScmpNotifResp {
             )),
             _ => return Err(Errno::EINVAL),
         };
-        #[allow(clippy::cast_possible_truncation)]
-        #[allow(clippy::cast_possible_wrap)]
+        #[expect(clippy::cast_possible_truncation)]
         let target_suid = match req.data.args[2] as i32 {
             -1 => None,
             n if n >= 0 => Some(Uid::from_raw(
@@ -543,7 +536,7 @@ pub(crate) fn sys_setresuid(request: UNotifyEventRequest) -> ScmpNotifResp {
             if target_ruid.as_raw() <= UID_MIN.as_raw() {
                 // SAFETY: This is already asserted with the parent
                 // seccomp-bpf filter, this is the second layer.
-                return Ok(request.fail_syscall(Errno::EACCES));
+                return Err(Errno::EACCES);
             } else if source_ruid != target_ruid {
                 change = true;
             }
@@ -552,7 +545,7 @@ pub(crate) fn sys_setresuid(request: UNotifyEventRequest) -> ScmpNotifResp {
             if target_euid.as_raw() <= UID_MIN.as_raw() {
                 // SAFETY: This is already asserted with the parent
                 // seccomp-bpf filter, this is the second layer.
-                return Ok(request.fail_syscall(Errno::EACCES));
+                return Err(Errno::EACCES);
             } else if source_euid != target_euid {
                 change = true;
             }
@@ -561,7 +554,7 @@ pub(crate) fn sys_setresuid(request: UNotifyEventRequest) -> ScmpNotifResp {
             if target_suid.as_raw() <= UID_MIN.as_raw() {
                 // SAFETY: This is already asserted with the parent
                 // seccomp-bpf filter, this is the second layer.
-                return Ok(request.fail_syscall(Errno::EACCES));
+                return Err(Errno::EACCES);
             } else if source_suid != target_suid {
                 change = true;
             }
@@ -573,13 +566,13 @@ pub(crate) fn sys_setresuid(request: UNotifyEventRequest) -> ScmpNotifResp {
         }
 
         let sandbox = request.get_sandbox();
-        let verbose = sandbox.verbose;
+        let log_scmp = sandbox.log_scmp();
 
         // SAFETY: We do not support RUID != EUID != SUID
         if let Some(target_ruid) = target_ruid {
             if let Some(target_euid) = target_euid {
                 if target_ruid != target_euid {
-                    if verbose {
+                    if log_scmp {
                         warn!("ctx": "safesetid", "err": libc::EACCES,
                             "sys": request.syscall, "req": &request,
                             "target_suid": target_suid.map(|u| u.as_raw()),
@@ -611,7 +604,7 @@ pub(crate) fn sys_setresuid(request: UNotifyEventRequest) -> ScmpNotifResp {
         if let Some(target_ruid) = target_ruid {
             if let Some(target_suid) = target_suid {
                 if target_ruid != target_suid {
-                    if verbose {
+                    if log_scmp {
                         warn!("ctx": "safesetid", "err": libc::EACCES,
                             "sys": request.syscall, "req": &request,
                             "target_suid": target_suid.as_raw(),
@@ -643,7 +636,7 @@ pub(crate) fn sys_setresuid(request: UNotifyEventRequest) -> ScmpNotifResp {
         if let Some(target_euid) = target_euid {
             if let Some(target_suid) = target_suid {
                 if target_euid != target_suid {
-                    if verbose {
+                    if log_scmp {
                         warn!("ctx": "safesetid", "err": libc::EACCES,
                             "sys": request.syscall, "req": &request,
                             "target_suid": target_suid.as_raw(),
@@ -699,7 +692,7 @@ pub(crate) fn sys_setresuid(request: UNotifyEventRequest) -> ScmpNotifResp {
         let target_euid = target_euid.map(|uid| i64::from(uid.as_raw())).unwrap_or(-1);
         let target_suid = target_suid.map(|uid| i64::from(uid.as_raw())).unwrap_or(-1);
         if !allowed {
-            if verbose {
+            if log_scmp {
                 warn!("ctx": "safesetid", "err": libc::EACCES,
                     "sys": request.syscall, "req": request,
                     "target_suid": target_suid,
@@ -731,7 +724,7 @@ pub(crate) fn sys_setresuid(request: UNotifyEventRequest) -> ScmpNotifResp {
         if let Err(errno) = Errno::result(unsafe {
             libc::syscall(libc::SYS_setresuid, target_ruid, target_euid, target_suid)
         }) {
-            if verbose {
+            if log_scmp {
                 warn!("ctx": "safesetid", "err": errno as i32,
                     "sys": request.syscall, "req": request,
                     "target_suid": target_suid,
@@ -771,13 +764,12 @@ pub(crate) fn sys_setresuid(request: UNotifyEventRequest) -> ScmpNotifResp {
     })
 }
 
-#[allow(clippy::cognitive_complexity)]
+#[expect(clippy::cognitive_complexity)]
 pub(crate) fn sys_setresgid(request: UNotifyEventRequest) -> ScmpNotifResp {
     syscall_handler!(request, |request: UNotifyEventRequest| {
         let req = request.scmpreq;
 
-        #[allow(clippy::cast_possible_truncation)]
-        #[allow(clippy::cast_possible_wrap)]
+        #[expect(clippy::cast_possible_truncation)]
         let target_rgid = match req.data.args[0] as i32 {
             -1 => None,
             n if n >= 0 => Some(Gid::from_raw(
@@ -785,8 +777,7 @@ pub(crate) fn sys_setresgid(request: UNotifyEventRequest) -> ScmpNotifResp {
             )),
             _ => return Err(Errno::EINVAL),
         };
-        #[allow(clippy::cast_possible_truncation)]
-        #[allow(clippy::cast_possible_wrap)]
+        #[expect(clippy::cast_possible_truncation)]
         let target_egid = match req.data.args[1] as i32 {
             -1 => None,
             n if n >= 0 => Some(Gid::from_raw(
@@ -794,8 +785,7 @@ pub(crate) fn sys_setresgid(request: UNotifyEventRequest) -> ScmpNotifResp {
             )),
             _ => return Err(Errno::EINVAL),
         };
-        #[allow(clippy::cast_possible_truncation)]
-        #[allow(clippy::cast_possible_wrap)]
+        #[expect(clippy::cast_possible_truncation)]
         let target_sgid = match req.data.args[2] as i32 {
             -1 => None,
             n if n >= 0 => Some(Gid::from_raw(
@@ -820,7 +810,7 @@ pub(crate) fn sys_setresgid(request: UNotifyEventRequest) -> ScmpNotifResp {
             if target_rgid.as_raw() <= GID_MIN.as_raw() {
                 // SAFETY: This is already asserted with the parent
                 // seccomp-bpf filter, this is the second layer.
-                return Ok(request.fail_syscall(Errno::EACCES));
+                return Err(Errno::EACCES);
             } else if source_rgid != target_rgid {
                 change = true;
             }
@@ -829,7 +819,7 @@ pub(crate) fn sys_setresgid(request: UNotifyEventRequest) -> ScmpNotifResp {
             if target_egid.as_raw() <= GID_MIN.as_raw() {
                 // SAFETY: This is already asserted with the parent
                 // seccomp-bpf filter, this is the second layer.
-                return Ok(request.fail_syscall(Errno::EACCES));
+                return Err(Errno::EACCES);
             } else if source_egid != target_egid {
                 change = true;
             }
@@ -838,7 +828,7 @@ pub(crate) fn sys_setresgid(request: UNotifyEventRequest) -> ScmpNotifResp {
             if target_sgid.as_raw() <= GID_MIN.as_raw() {
                 // SAFETY: This is already asserted with the parent
                 // seccomp-bpf filter, this is the second layer.
-                return Ok(request.fail_syscall(Errno::EACCES));
+                return Err(Errno::EACCES);
             } else if source_sgid != target_sgid {
                 change = true;
             }
@@ -850,13 +840,13 @@ pub(crate) fn sys_setresgid(request: UNotifyEventRequest) -> ScmpNotifResp {
         }
 
         let sandbox = request.get_sandbox();
-        let verbose = sandbox.verbose;
+        let log_scmp = sandbox.log_scmp();
 
         // SAFETY: We do not support Rgid != Egid != Sgid
         if let Some(target_rgid) = target_rgid {
             if let Some(target_egid) = target_egid {
                 if target_rgid != target_egid {
-                    if verbose {
+                    if log_scmp {
                         warn!("ctx": "safesetid", "err": libc::EACCES,
                             "sys": request.syscall, "req": &request,
                             "target_sgid": target_sgid.map(|u| u.as_raw()),
@@ -888,7 +878,7 @@ pub(crate) fn sys_setresgid(request: UNotifyEventRequest) -> ScmpNotifResp {
         if let Some(target_rgid) = target_rgid {
             if let Some(target_sgid) = target_sgid {
                 if target_rgid != target_sgid {
-                    if verbose {
+                    if log_scmp {
                         warn!("ctx": "safesetid", "err": libc::EACCES,
                             "sys": request.syscall, "req": &request,
                             "target_sgid": target_sgid.as_raw(),
@@ -920,7 +910,7 @@ pub(crate) fn sys_setresgid(request: UNotifyEventRequest) -> ScmpNotifResp {
         if let Some(target_egid) = target_egid {
             if let Some(target_sgid) = target_sgid {
                 if target_egid != target_sgid {
-                    if verbose {
+                    if log_scmp {
                         warn!("ctx": "safesetid", "err": libc::EACCES,
                             "sys": request.syscall, "req": &request,
                             "target_sgid": target_sgid.as_raw(),
@@ -976,7 +966,7 @@ pub(crate) fn sys_setresgid(request: UNotifyEventRequest) -> ScmpNotifResp {
         let target_egid = target_egid.map(|gid| i64::from(gid.as_raw())).unwrap_or(-1);
         let target_sgid = target_sgid.map(|gid| i64::from(gid.as_raw())).unwrap_or(-1);
         if !allowed {
-            if verbose {
+            if log_scmp {
                 warn!("ctx": "safesetid", "err": libc::EACCES,
                     "sys": request.syscall, "req": request,
                     "target_sgid": target_sgid,
@@ -1008,7 +998,7 @@ pub(crate) fn sys_setresgid(request: UNotifyEventRequest) -> ScmpNotifResp {
         if let Err(errno) = Errno::result(unsafe {
             libc::syscall(libc::SYS_setresgid, target_rgid, target_egid, target_sgid)
         }) {
-            if verbose {
+            if log_scmp {
                 warn!("ctx": "safesetid", "err": errno as i32,
                     "sys": request.syscall, "req": request,
                     "target_sgid": target_sgid,
