@@ -12,8 +12,8 @@ use libseccomp::ScmpNotifResp;
 use nix::errno::Errno;
 
 use crate::{
-    config::MMAP_MIN_ADDR,
-    fs::is_valid_fd,
+    confine::is_valid_ptr,
+    fd::is_valid_fd,
     kernel::syscall_path_handler,
     lookup::FsFlags,
     req::{SysArg, UNotifyEventRequest},
@@ -22,7 +22,7 @@ use crate::{
 pub(crate) fn sys_statfs(request: UNotifyEventRequest) -> ScmpNotifResp {
     // SAFETY: If second argument is not a valid pointer, return EFAULT.
     let req = request.scmpreq;
-    if req.data.args[1] < *MMAP_MIN_ADDR {
+    if !is_valid_ptr(req.data.args[1], req.data.arch) {
         return request.fail_syscall(Errno::EFAULT);
     }
     let argv = &[SysArg {
@@ -56,7 +56,7 @@ pub(crate) fn sys_statfs(request: UNotifyEventRequest) -> ScmpNotifResp {
 
         let result =
                 // SAFETY: Libc version may call fstatfs64 behind our back!
-                Errno::result(unsafe { libc::syscall(libc::SYS_fstatfs, fd.as_raw_fd(), ptr) });
+                Errno::result(unsafe { libc::syscall(crate::compat::SYS_fstatfs, fd.as_raw_fd(), ptr) });
 
         // Remove invalidation record unless interrupted.
         request
@@ -80,7 +80,7 @@ pub(crate) fn sys_statfs64(request: UNotifyEventRequest) -> ScmpNotifResp {
         Ok(siz) => siz,
         Err(_) => return request.fail_syscall(Errno::EINVAL),
     };
-    if req.data.args[2] < *MMAP_MIN_ADDR {
+    if !is_valid_ptr(req.data.args[2], req.data.arch) {
         return request.fail_syscall(Errno::EFAULT);
     }
     let argv = &[SysArg {
@@ -135,7 +135,7 @@ pub(crate) fn sys_fstatfs(request: UNotifyEventRequest) -> ScmpNotifResp {
     let req = request.scmpreq;
     if !is_valid_fd(req.data.args[0]) {
         return request.fail_syscall(Errno::EBADF);
-    } else if req.data.args[1] < *MMAP_MIN_ADDR {
+    } else if !is_valid_ptr(req.data.args[1], req.data.arch) {
         return request.fail_syscall(Errno::EFAULT);
     }
 
@@ -164,7 +164,7 @@ pub(crate) fn sys_fstatfs(request: UNotifyEventRequest) -> ScmpNotifResp {
 
         let result =
             // SAFETY: Libc version may call fstatfs64 behind our back!
-            Errno::result(unsafe { libc::syscall(libc::SYS_fstatfs, fd.as_raw_fd(), ptr) });
+            Errno::result(unsafe { libc::syscall(crate::compat::SYS_fstatfs, fd.as_raw_fd(), ptr) });
 
         // Remove invalidation record unless interrupted.
         request
@@ -194,7 +194,7 @@ pub(crate) fn sys_fstatfs64(request: UNotifyEventRequest) -> ScmpNotifResp {
     if !is_valid_fd(req.data.args[0]) {
         return request.fail_syscall(Errno::EBADF);
     }
-    if req.data.args[2] < *MMAP_MIN_ADDR {
+    if !is_valid_ptr(req.data.args[2], req.data.arch) {
         return request.fail_syscall(Errno::EFAULT);
     }
 
