@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/pool.rs: Self growing / shrinking `ThreadPool` implementation
 //
-// Copyright (c) 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 // Based in part upon rusty_pool which is:
 //     Copyright (c) Robin Friedli <robinfriedli@icloud.com>
 //     SPDX-License-Identifier: Apache-2.0
@@ -124,7 +124,8 @@ impl ThreadPool {
             Some(ExportMode::BerkeleyPacketFilter) => {
                 // Worker rules
                 let is_crypt = self.cache.crypt_map.is_some();
-                let ctx = Worker::prepare_confine(self.seccomp_fd, self.flags, is_crypt, &[], &[])?;
+                let ctx =
+                    Worker::prepare_confine(self.seccomp_fd, self.flags, is_crypt, None, &[], &[])?;
                 let file = OpenOptions::new()
                     .write(true)
                     .create_new(true)
@@ -161,7 +162,7 @@ impl ThreadPool {
                 ctx.export_bpf(file)?;
 
                 // Aes worker rules
-                let ctx = AesWorker::prepare_confine(self.flags, &[], &[])?;
+                let ctx = AesWorker::prepare_confine(self.flags, &[], &[], true)?;
                 let file = OpenOptions::new()
                     .write(true)
                     .create_new(true)
@@ -179,7 +180,8 @@ impl ThreadPool {
                     self.seccomp_fd
                 )?;
                 let is_crypt = self.cache.crypt_map.is_some();
-                let ctx = Worker::prepare_confine(self.seccomp_fd, self.flags, is_crypt, &[], &[])?;
+                let ctx =
+                    Worker::prepare_confine(self.seccomp_fd, self.flags, is_crypt, None, &[], &[])?;
                 rwrite!(stdout, "{}", seccomp_export_pfc(&ctx)?)?;
 
                 // We pass dry_run=true to avoid Landlock confinement.
@@ -205,7 +207,7 @@ impl ThreadPool {
                 rwrite!(stdout, "{}", seccomp_export_pfc(&ctx)?)?;
 
                 rwriteln!(stdout, "# Syd encryptor rules")?;
-                let ctx = AesWorker::prepare_confine(self.flags, &[], &[])?;
+                let ctx = AesWorker::prepare_confine(self.flags, &[], &[], true)?;
                 rwrite!(stdout, "{}", seccomp_export_pfc(&ctx)?)?;
             }
             _ => {}
@@ -316,19 +318,28 @@ impl ThreadPool {
                 //    as it is inherited.
                 // 2. If sandbox is not locked yet, build the seccomp context anyway,
                 //    precompute it and pass it to emulator threads for fast confinement.
+                // 3. If sandbox is locked, and trace/force_umask is set, use it to confine
+                //    fchmodat(2) and fchmodat2(2) mode argument.
                 let dry_run = env::var_os(ENV_SKIP_SCMP).is_some() || ExportMode::from_env().is_some();
                 let safe_setid = self.flags.intersects(Flags::FL_ALLOW_SAFE_SETUID | Flags::FL_ALLOW_SAFE_SETGID);
                 let is_crypt = self.cache.crypt_map.is_some();
+                let is_lock = sandbox.locked();
+                let force_umask = if is_lock {
+                    sandbox.umask
+                } else {
+                    None
+                };
 
                 let mut ctx = if !dry_run {
                     let ctx = Worker::prepare_confine(
                         self.seccomp_fd,
                         self.flags,
                         is_crypt,
+                        force_umask,
                         &sandbox.transit_uids,
                         &sandbox.transit_gids)?;
 
-                    if Sandbox::locked_once() {
+                    if is_lock {
                         // Sandbox locked, confine right away.
                         //
                         // SAFETY: We use exit_group(2) here to bail,
@@ -657,6 +668,7 @@ impl ThreadPool {
     fn signal_int(&self) {
         // Unblock stuck emulator threads with manual signal.
         let mut nsig = 0usize;
+
         {
             let (ref lock, ref cvar) = *self.cache.sysint_map.sys_block;
             let mut map = lock.lock().unwrap_or_else(|err| err.into_inner());
@@ -672,10 +684,12 @@ impl ThreadPool {
             cvar.notify_one();
         }
 
-        alert!("ctx": "spawn",
-            "msg": format!("signaled interrupt to unstuck {nsig} emulators"),
-            "nsig": nsig,
-            "core_size": self.core_size,
-            "keep_alive": self.keep_alive);
+        if nsig > 0 {
+            alert!("ctx": "spawn",
+                "msg": format!("signaled interrupt to unstuck {nsig} emulators"),
+                "nsig": nsig,
+                "core_size": self.core_size,
+                "keep_alive": self.keep_alive);
+        }
     }
 }
