@@ -25,10 +25,11 @@
 #![deny(unsafe_code)]
 
 use std::{
+    borrow::Cow,
     ffi::{CStr, OsStr, OsString},
     fmt,
     fs::File,
-    io::{BufReader, Cursor, IoSlice, IoSliceMut, Read, Seek, SeekFrom},
+    io::{BufReader, Cursor, IoSlice, IoSliceMut, Read, Seek, SeekFrom, Write},
     num::NonZeroUsize,
     ops::Range,
     os::{
@@ -63,17 +64,17 @@ use procfs_core::{
 use serde::{ser::SerializeMap, Serialize, Serializer};
 
 use crate::{
-    compat::{getdents64, openat2, statx, ResolveFlag, STATX_INO},
+    compat::{getdents64, openat2, statx, OpenHow, ResolveFlag, STATX_INO},
     config::*,
     confine::SydMemoryMap,
     cookie::{CookieIdx, SYSCOOKIE_POOL},
     elf::ElfType,
     err::{err2no, proc_error_to_errno},
-    fd::{is_empty_file, is_open_fd, parse_fd, AT_BADFD},
+    fd::{is_empty_file, is_open_fd, is_proc, parse_fd, AT_BADFD},
     fs::readlinkat,
     hash::{SydHashMap, SydHashSet, SydIndexSet},
     io::{read_all, read_buf, write_all},
-    lookup::{safe_open, safe_open_how_abs},
+    lookup::safe_open,
     parsers::{
         map_result,
         proc::{
@@ -887,10 +888,19 @@ pub fn proc_pipemax_read<Fd: AsFd>(fd: Fd) -> Result<libc::c_int, Errno> {
 
 /// Retrieves the value of /proc/sys/fs/file-max.
 pub fn proc_fs_file_max() -> Result<u64, Errno> {
-    // SAFETY: Informational command,
-    // runs before syd::config::proc_init.
-    #[expect(clippy::disallowed_methods)]
-    let fd = nix::fcntl::openat(AT_BADFD, "/proc/sys/fs/file-max", OFLAG_PROC, Mode::empty())?;
+    let fd = proc_open().and_then(|fd| {
+        safe_open(
+            fd,
+            c"sys/fs/file/max",
+            OFlag::O_RDONLY | OFlag::O_NOCTTY,
+            ResolveFlag::RESOLVE_NO_XDEV,
+        )
+    })?;
+
+    // Check the file is an empty file or bail out.
+    if !is_empty_file(&fd).unwrap_or(false) {
+        return Err(Errno::EBADFD);
+    }
 
     // Read up to 24 bytes + 1 bytes for \n.
     let mut data = [0u8; 25];
@@ -901,10 +911,19 @@ pub fn proc_fs_file_max() -> Result<u64, Errno> {
 
 /// Retrieves the value of /proc/sys/fs/nr_open.
 pub fn proc_fs_nr_open() -> Result<u64, Errno> {
-    // SAFETY: Informational command,
-    // runs before syd::config::proc_init.
-    #[expect(clippy::disallowed_methods)]
-    let fd = nix::fcntl::openat(AT_BADFD, "/proc/sys/fs/nr_open", OFLAG_PROC, Mode::empty())?;
+    let fd = proc_open().and_then(|fd| {
+        safe_open(
+            fd,
+            c"sys/fs/nr_open",
+            OFlag::O_RDONLY | OFlag::O_NOCTTY,
+            ResolveFlag::RESOLVE_NO_XDEV,
+        )
+    })?;
+
+    // Check the file is an empty file or bail out.
+    if !is_empty_file(&fd).unwrap_or(false) {
+        return Err(Errno::EBADFD);
+    }
 
     // Read up to 24 bytes + 1 bytes for \n.
     let mut data = [0u8; 25];
@@ -915,15 +934,19 @@ pub fn proc_fs_nr_open() -> Result<u64, Errno> {
 
 /// Retrieves the value of /proc/sys/vm/mmap_min_addr.
 pub fn proc_mmap_min_addr() -> Result<u64, Errno> {
-    // SAFETY: Informational command,
-    // runs before syd::config::proc_init.
-    #[expect(clippy::disallowed_methods)]
-    let fd = nix::fcntl::openat(
-        AT_BADFD,
-        "/proc/sys/vm/mmap_min_addr",
-        OFLAG_PROC,
-        Mode::empty(),
-    )?;
+    let fd = proc_open().and_then(|fd| {
+        safe_open(
+            fd,
+            c"sys/vm/mmap_min_addr",
+            OFlag::O_RDONLY | OFlag::O_NOCTTY,
+            ResolveFlag::RESOLVE_NO_XDEV,
+        )
+    })?;
+
+    // Check the file is an empty file or bail out.
+    if !is_empty_file(&fd).unwrap_or(false) {
+        return Err(Errno::EBADFD);
+    }
 
     // Read up to 24 bytes + 1 bytes for \n.
     let mut data = [0u8; 25];
@@ -932,33 +955,21 @@ pub fn proc_mmap_min_addr() -> Result<u64, Errno> {
     btoi::<u64>(data[..nread].trim_ascii()).or(Err(Errno::EINVAL))
 }
 
-/// Sets the value of /proc/sys/kernel/pid_max.
-pub fn proc_pid_max(max: u64) -> Result<(), Errno> {
-    // SAFETY: System configuration command,
-    // runs during syd::config::proc_init.
-    #[expect(clippy::disallowed_methods)]
-    let fd = nix::fcntl::openat(
-        AT_BADFD,
-        "/proc/sys/kernel/pid_max",
-        OFLAG_PROC | OFlag::O_WRONLY,
-        Mode::empty(),
-    )?;
-
-    let mut buf = itoa::Buffer::new();
-    write_all(fd, buf.format(max).as_bytes())
-}
-
 /// Retrieves the value of /proc/sys/kernel/randomize_va_space
 pub fn proc_kernel_randomize_va_space() -> Result<u8, Errno> {
-    // SAFETY: Informational command,
-    // runs before syd::config::proc_init.
-    #[expect(clippy::disallowed_methods)]
-    let fd = nix::fcntl::openat(
-        AT_BADFD,
-        "/proc/sys/kernel/randomize_va_space",
-        OFLAG_PROC,
-        Mode::empty(),
-    )?;
+    let fd = proc_open().and_then(|fd| {
+        safe_open(
+            fd,
+            c"sys/kernel/randomize_va_space",
+            OFlag::O_RDONLY | OFlag::O_NOCTTY,
+            ResolveFlag::RESOLVE_NO_XDEV,
+        )
+    })?;
+
+    // Check the file is an empty file or bail out.
+    if !is_empty_file(&fd).unwrap_or(false) {
+        return Err(Errno::EBADFD);
+    }
 
     // Read up to 1 byte + 1 byte for \n.
     let mut data = [0u8; 2];
@@ -974,12 +985,12 @@ pub fn proc_kernel_randomize_va_space() -> Result<u8, Errno> {
 pub fn proc_tty(pid: Pid) -> Result<XPathBuf, Errno> {
     let stat = proc_stat(pid)?;
     if stat.tty_nr <= 0 {
-        // Process has no controlling terminal
+        // Process has no controlling terminal.
         return Err(Errno::ENXIO);
     }
 
-    // minor is bits 31-20 and 7-0
-    // major is 15-8
+    // minor is bits 31-20 and 7-0.
+    // major is 15-8.
     let tty_nr = stat.tty_nr;
 
     // mmmmmmmmmmmm____MMMMMMMMmmmmmmmm
@@ -2230,15 +2241,19 @@ impl fmt::Display for KernelTaintFlags {
 
 /// Retrieves the value of /proc/sys/kernel/tainted as KernelTaintFlags.
 pub fn proc_kernel_taint() -> Result<KernelTaintFlags, Errno> {
-    // SAFETY: Informational command,
-    // runs before syd::config::proc_init.
-    #[expect(clippy::disallowed_methods)]
-    let fd = nix::fcntl::openat(
-        AT_BADFD,
-        "/proc/sys/kernel/tainted",
-        OFLAG_PROC,
-        Mode::empty(),
-    )?;
+    let fd = proc_open().and_then(|fd| {
+        safe_open(
+            fd,
+            c"sys/kernel/tainted",
+            OFlag::O_RDONLY | OFlag::O_NOCTTY,
+            ResolveFlag::RESOLVE_NO_XDEV,
+        )
+    })?;
+
+    // Check the file is an empty file or bail out.
+    if !is_empty_file(&fd).unwrap_or(false) {
+        return Err(Errno::EBADFD);
+    }
 
     // Read up to 24 bytes + 1 byte for \n.
     let mut data = [0u8; 25];
@@ -2249,17 +2264,90 @@ pub fn proc_kernel_taint() -> Result<KernelTaintFlags, Errno> {
     Ok(KernelTaintFlags::from_bits_retain(val))
 }
 
-/// Write the given $data to /proc/$pid/$name.
-pub fn proc_write(pid: Pid, name: &[u8], data: &[u8]) -> Result<(), Errno> {
-    let mut pfd = XPathBuf::from("/proc");
-    pfd.push_pid(pid);
-    pfd.push(name);
+/// Set max PID limit for PID namespace.
+pub fn proc_pid_max<Fd: AsFd>(fd_proc: Fd, pid_max: u64) -> Result<(), Errno> {
+    let mut buf = itoa::Buffer::new();
+    let pid_max = buf.format(pid_max).as_bytes();
+    proc_write(&fd_proc, b"sys/kernel/pid/max", pid_max, false /*pid*/)
+}
 
-    let how = safe_open_how_abs(OFlag::O_WRONLY, ResolveFlag::empty());
+/// Set up user mappings for user namespace.
+pub fn proc_map_user<Fd: AsFd>(
+    fd_proc: Fd,
+    uid: Uid,
+    gid: Gid,
+    map_root: bool,
+) -> Result<(), Errno> {
+    let uid_map = {
+        let mut buf = Vec::new();
+        write!(
+            &mut buf,
+            "{} {uid} 1",
+            if map_root { 0 } else { uid.as_raw() }
+        )
+        .map_err(|err| err2no(&err))?;
+        buf
+    };
+    let gid_map = {
+        let mut buf = Vec::new();
+        write!(
+            &mut buf,
+            "{} {gid} 1",
+            if map_root { 0 } else { gid.as_raw() }
+        )
+        .map_err(|err| err2no(&err))?;
+        buf
+    };
 
-    // This function runs early at startup before syd::config::proc_init.
-    #[expect(clippy::disallowed_methods)]
-    let fd = retry_on_eintr(|| openat2(AT_BADFD, &pfd, how))?;
+    // Write "deny" to /proc/self/setgroups before writing to gid_map.
+    proc_write(&fd_proc, b"setgroups", b"deny", true /*pid*/)?;
+
+    // Write GID mapping to gid_map.
+    proc_write(&fd_proc, b"gid_map", &gid_map, true /*pid*/)?;
+
+    // Write UID mapping to uid map.
+    proc_write(&fd_proc, b"uid_map", &uid_map, true /*pid*/)?;
+
+    Ok(())
+}
+
+/// Set up time offsets for time namespace.
+pub fn proc_set_time<Fd: AsFd>(fd_proc: Fd, boffset: i64, moffset: i64) -> Result<(), Errno> {
+    // <clock-id> <offset-secs> <offset-nanosecs>
+    let mut buf = itoa::Buffer::new();
+    let mut data = "monotonic ".to_string();
+    data.push_str(buf.format(moffset));
+    data.push_str(" 0\nboottime ");
+    data.push_str(buf.format(boffset));
+    data.push_str(" 0\n");
+
+    proc_write(
+        fd_proc,
+        b"timens_offsets",
+        data.as_bytes(),
+        true, /*pid*/
+    )
+}
+
+// Write the given $data to /proc/$pid/$name.
+fn proc_write<Fd: AsFd>(fd_proc: Fd, name: &[u8], data: &[u8], pid: bool) -> Result<(), Errno> {
+    let pfd = if pid {
+        let mut pfd = XPathBuf::from_pid(Pid::this())?;
+        pfd.try_reserve(name.len().checked_add(1).ok_or(Errno::EOVERFLOW)?)
+            .or(Err(Errno::ENOMEM))?;
+        pfd.push(name);
+        Cow::Owned(pfd)
+    } else {
+        Cow::Borrowed(XPath::from_bytes(name))
+    };
+
+    // Open proc file safely.
+    let fd = safe_open(
+        fd_proc,
+        pfd.as_ref(),
+        OFlag::O_WRONLY,
+        ResolveFlag::RESOLVE_NO_XDEV,
+    )?;
 
     // Check the file is an empty file or bail out.
     if !is_empty_file(&fd).unwrap_or(false) {
@@ -2269,6 +2357,23 @@ pub fn proc_write(pid: Pid, name: &[u8], data: &[u8]) -> Result<(), Errno> {
     write_all(&fd, data)
 }
 
+/// Open /proc safely and validate procfs.
+pub fn proc_open() -> Result<OwnedFd, Errno> {
+    let how = OpenHow::new()
+        .flags(OFlag::O_PATH | OFlag::O_DIRECTORY | OFlag::O_NOFOLLOW | OFlag::O_CLOEXEC)
+        .resolve(ResolveFlag::RESOLVE_NO_MAGICLINKS | ResolveFlag::RESOLVE_NO_SYMLINKS);
+    // This function is called before syd::config::proc_init.
+    #[expect(clippy::disallowed_methods)]
+    let fd_proc = openat2(AT_BADFD, "/proc", how)?;
+
+    // SAFETY: Validate what we've opened is procfs(5).
+    if !is_proc(&fd_proc).unwrap_or(false) {
+        return Err(Errno::ENODEV);
+    }
+
+    Ok(fd_proc)
+}
+
 #[cfg(test)]
 mod tests {
     use std::os::unix::fs::PermissionsExt;
