@@ -50,7 +50,10 @@ use crate::{
     config::{MMAP_MIN_ADDR, PAGE_SIZE, PROC_FILE},
     confine::{is_valid_ptr, op2name, scmp_arch_bits, ScmpNotifReq, SydArch, Sydcall, EIDRM},
     error,
-    fd::{fd_status_flags, pidfd_getfd, pidfd_open, pidfd_send_signal, to_valid_fd, PIDFD_THREAD},
+    fd::{
+        fd_inode, fd_status_flags, peer_inode, pidfd_getfd, pidfd_open, pidfd_send_signal,
+        to_valid_fd, PIDFD_THREAD,
+    },
     fs::{process_mrelease, seccomp_notify_addfd, seccomp_notify_id_valid},
     lookup::{file_type, safe_canonicalize, safe_open_msym, CanonicalPath, FileType, FsFlags},
     path::{XPath, XPathBuf, PATH_MAX, PATH_MIN},
@@ -407,8 +410,7 @@ impl RemoteProcess {
                 self.pid,
                 canonical_path.typ.as_ref(),
                 None,
-                !sandbox.flags.allow_unsafe_filename(),
-                !sandbox.flags.allow_unsafe_mkbdev(),
+                sandbox.flags.deref().into(),
             )?;
         }
 
@@ -1028,6 +1030,22 @@ impl UNotifyEventRequest {
         SandboxGuard::Write(self.sandbox.write().unwrap_or_else(|err| err.into_inner()))
     }
 
+    /// Fix PID in SO_PASSCRED UNIX credentials message.
+    pub(crate) fn fix_cred_pid<Fd: AsFd>(&self, sock: Fd) -> Pid {
+        // Determine peer PID, on errors return caller PID for safety.
+        // Get socket inode, and lookup pid by inode in sandbox unix map.
+        if let Ok(inode) = fd_inode(sock) {
+            if let Ok(ipeer) = peer_inode(inode) {
+                if let Some(unix) = self.get_unix(ipeer) {
+                    return unix.pid;
+                }
+            } else if let Some(unix) = self.get_unix(inode) {
+                return unix.pid;
+            }
+        }
+        self.scmpreq.pid()
+    }
+
     /// Check SCM_RIGHTS file types at sendm{,m}sg(2) boundaries.
     #[expect(clippy::cognitive_complexity)]
     pub(crate) fn check_scm_rights<Fd: AsFd>(
