@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/rng.rs: OS Random Number Generator (RNG) interface
 //
-// Copyright (c) 2023, 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2023, 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 //
 // SPDX-License-Identifier: GPL-3.0
 
@@ -13,7 +13,7 @@ use std::{
     os::fd::{AsFd, AsRawFd, BorrowedFd, OwnedFd, RawFd},
 };
 
-use libc::{c_int, GRND_RANDOM};
+use libc::GRND_RANDOM;
 use memchr::memchr;
 use nix::{
     errno::Errno,
@@ -25,32 +25,37 @@ use nix::{
 use crate::{
     compat::{dup3, ResolveFlag},
     cookie::safe_unlinkat,
-    fs::{close, is_active_fd},
+    fd::{close, is_active_fd},
     log::{now, Tm},
     lookup::safe_open,
     path::{XPathBuf, PATH_MAX},
     retry::retry_on_eintr,
 };
 
-/// RAII guard that disables pthread cancellation for the current thread
-/// and restores the previous state on drop. Uses pthread_setcancelstate(3).
+// RAII guard that disables pthread cancellation for the current thread
+// and restores the previous state on drop. Uses pthread_setcancelstate(3).
+#[cfg(not(target_os = "android"))]
 #[must_use = "hold the guard to keep cancellation disabled"]
-pub struct CancelGuard(c_int);
+struct CancelGuard(libc::c_int);
 
-const _PTHREAD_CANCEL_ENABLE: c_int = 0;
-const PTHREAD_CANCEL_DISABLE: c_int = 1;
+#[cfg(not(target_os = "android"))]
+const _PTHREAD_CANCEL_ENABLE: libc::c_int = 0;
+#[cfg(not(target_os = "android"))]
+const PTHREAD_CANCEL_DISABLE: libc::c_int = 1;
 
 // Libc crate does not define this symbol explicitly yet.
+#[cfg(not(target_os = "android"))]
 extern "C" {
-    fn pthread_setcancelstate(state: c_int, oldstate: *mut c_int) -> c_int;
+    fn pthread_setcancelstate(state: libc::c_int, oldstate: *mut libc::c_int) -> libc::c_int;
 }
 
+#[cfg(not(target_os = "android"))]
 impl CancelGuard {
     /// Acquire the guard by disabling pthread cancellation for this thread.
     ///
     /// Returns a guard that will restore the previous state when dropped.
     pub fn acquire() -> Result<Self, Errno> {
-        let mut old: c_int = 0;
+        let mut old: libc::c_int = 0;
 
         // SAFETY: We call pthread_setcancelstate(3) for the current thread.
         // - PTHREAD_CANCEL_DISABLE is a valid constant.
@@ -67,6 +72,7 @@ impl CancelGuard {
     }
 }
 
+#[cfg(not(target_os = "android"))]
 impl Drop for CancelGuard {
     fn drop(&mut self) {
         // SAFETY: Restore the exact state captured at construction
@@ -188,9 +194,7 @@ pub fn getrandom(size: usize) -> Result<Vec<u8>, Errno> {
     }
 
     let mut buf = Vec::new();
-    if buf.try_reserve(size).is_err() {
-        return Err(Errno::ENOMEM);
-    }
+    buf.try_reserve(size).or(Err(Errno::ENOMEM))?;
     buf.resize(size, 0);
 
     fillrandom(&mut buf)?;
@@ -208,6 +212,7 @@ pub fn fillrandom(buf: &mut [u8]) -> Result<(), Errno> {
 
     // Disable pthread cancellation within this critical section.
     // Restored automatically when guard is dropped.
+    #[cfg(not(target_os = "android"))]
     let guard = CancelGuard::acquire()?;
 
     let mut n = 0;
@@ -229,6 +234,7 @@ pub fn fillrandom(buf: &mut [u8]) -> Result<(), Errno> {
     }
 
     // End of critical section.
+    #[cfg(not(target_os = "android"))]
     drop(guard);
 
     Ok(())
@@ -239,7 +245,7 @@ pub fn fillrandom(buf: &mut [u8]) -> Result<(), Errno> {
 /// Valid flags:
 /// - O_EXCL: closes oldfd after successful duplication.
 /// - All other flags are passed to dup3(2), ie O_CLOEXEC.
-pub fn duprand(oldfd: RawFd, mut flags: OFlag) -> Result<RawFd, Errno> {
+pub fn duprand(oldfd: RawFd, mut flags: OFlag) -> Result<OwnedFd, Errno> {
     let range_start = 7u64;
     let (range_end, _) = getrlimit(Resource::RLIMIT_NOFILE)?;
     #[expect(clippy::unnecessary_cast)]
@@ -274,12 +280,12 @@ pub fn duprand(oldfd: RawFd, mut flags: OFlag) -> Result<RawFd, Errno> {
             continue;
         }
 
-        match retry_on_eintr(|| dup3(oldfd, fd_rand.as_raw_fd(), flags.bits())) {
-            Ok(_) => {
+        match dup3(oldfd, fd_rand.as_raw_fd(), flags.bits()) {
+            Ok(fd_rand) => {
                 if close_old {
                     let _ = close(oldfd);
                 }
-                return Ok(fd_rand.as_raw_fd());
+                return Ok(fd_rand);
             }
             Err(Errno::EMFILE) => return Err(Errno::EMFILE),
             Err(_) => {}
