@@ -768,8 +768,12 @@ syd::main! {
     let ssb = sandbox.flags.allow_unsafe_exec_speculative();
 
     // Prepare stack for new Syd process.
-    #[expect(clippy::cast_possible_truncation)]
-    let mut stack = [0u8; MAIN_STACK_SIZE as usize];
+    // SAFETY: Heap-allocate the clone stack to avoid caller's
+    // stack frame, which may overflow RLIMIT_STACK.
+    let mut stack = Vec::new();
+    let stack_siz = MAIN_STACK_SIZE.try_into().or(Err(Errno::EOVERFLOW))?;
+    stack.try_reserve(stack_siz).or(Err(Errno::ENOMEM))?;
+    stack.resize(stack_siz, 0);
 
     struct SydChildInfo {
         sandbox: Sandbox,
