@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/test/main.rs: Run integration tests with TAP output
 //
-// Copyright (c) 2023, 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2023, 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 //
 // SPDX-License-Identifier: GPL-3.0
 
@@ -37,6 +37,7 @@ use test::*;
     not(coverage),
     not(feature = "prof"),
     not(target_os = "android"),
+    not(target_arch = "riscv64"),
     target_page_size_4k,
     target_pointer_width = "64"
 ))]
@@ -301,14 +302,13 @@ fn main() -> SydResult<ExitCode> {
         eprintln!("# Test seed: {seed}, set SYD_TEST_SEED to override.");
     }
     eprintln!("# Shuffling tests using the Fischer-Yates algorithm...");
-    unsafe { nix::libc::srand(seed) };
+    #[cfg(not(target_os = "android"))]
+    unsafe {
+        libc::srand(seed)
+    };
     shuffle_vec(&mut test_indices);
 
     // Print TAP plan.
-    if syd::confine::syd_enabled() {
-        println!("1..0 # SKIP syd is enabled!");
-        return Ok(ExitCode::from(0));
-    }
     let ntest = test_indices.len();
     let ptest = if env::var_os("SYD_TEST_REEXEC").is_none() {
         ""
