@@ -10,6 +10,7 @@ use libseccomp::ScmpFilterContext;
 use nix::{
     errno::Errno,
     fcntl::OFlag,
+    sched::CloneFlags,
     sys::{
         ptrace::{cont, Options},
         signal::{kill, Signal},
@@ -19,13 +20,13 @@ use nix::{
 };
 
 use crate::{
-    compat::{clone3, waitid, CloneArgs, WaitStatus, PTRACE_SEIZE},
-    fd::pidfd_getfd,
+    compat::{waitid, WaitStatus, PTRACE_SEIZE},
+    fd::{fdclone, pidfd_getfd},
     log::LOG_FD,
     retry::retry_on_eintr,
     rng::duprand,
     sandbox::RawIoctlMap,
-    unshare::{child, config::Config, Child, Command},
+    unshare::{child::child_after_clone, config::Config, Child, Command},
 };
 
 type ChildPreExecFunc = Box<dyn Fn() -> Result<(), Errno>>;
@@ -63,6 +64,7 @@ impl Command {
             )
         };
 
+        // Prepare information for the Syd child.
         let child_info = Box::new(ChildInfo {
             exe_file,
             exe_args,
@@ -75,26 +77,23 @@ impl Command {
         });
         let child_info_ptr: *mut libc::c_void = Box::into_raw(child_info) as *mut libc::c_void;
 
-        let mut pid_fd: libc::c_int = -1;
-        let mut clone_args = CloneArgs {
-            flags: libc::CLONE_PIDFD as u64,
-            pidfd: (&raw mut pid_fd) as u64,
-            exit_signal: libc::SIGCHLD as u64,
-            ..Default::default()
-        };
+        // Prepare stack for Syd child process.
+        let mut stack = [0u8; 1024];
 
-        let child = match clone3(&mut clone_args) {
-            Ok(child) => child,
-            Err(errno) => {
-                // SAFETY: Reconstruct and drop.
-                drop(unsafe { Box::from_raw(child_info_ptr as *mut ChildInfo) });
-                return Err(errno);
-            }
-        };
-
-        if child.as_raw() == 0 {
-            child::child_after_clone(child_info_ptr);
+        // SAFETY: Use clone(2) with CLONE_PIDFD to avoid pid recycling.
+        let (pid_fd, child) = unsafe {
+            fdclone(
+                child_after_clone,
+                &mut stack,
+                child_info_ptr,
+                CloneFlags::empty(),
+                Some(libc::SIGCHLD),
+            )
         }
+        .inspect_err(|_| {
+            // SAFETY: Reconstruct and drop parent's copy.
+            drop(unsafe { Box::from_raw(child_info_ptr as *mut ChildInfo) });
+        })?;
 
         // SAFETY: Put Syd into a new process group for signal safety.
         // This is best-effort as we may get EPERM here, e.g. under
@@ -113,8 +112,9 @@ impl Command {
         let _ = child_info.seccomp_pipefd.1 .1.into_raw_fd();
 
         // SAFETY: Randomize the pid FD for hardening.
-        // O_EXCL closes oldfd on success.
-        let pid_fd = duprand(pid_fd, OFlag::O_CLOEXEC | OFlag::O_EXCL)?;
+        let pid_fd_rand = duprand(pid_fd.as_raw_fd(), OFlag::O_CLOEXEC)?;
+        drop(pid_fd);
+        let pid_fd = pid_fd_rand;
 
         // SAFETY: Randomize the log FD for hardening.
         // O_EXCL closes oldfd on success.
