@@ -1,7 +1,7 @@
 // Syd: rock-solid application kernel
 // src/utils/syd-bit.rs: Utility to flip bits in a file
 //
-// Copyright (c) 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 //
 // SPDX-License-Identifier: GPL-3.0
 
@@ -22,6 +22,8 @@ use syd::{
 #[cfg(all(
     not(coverage),
     not(feature = "prof"),
+    not(target_os = "android"),
+    not(target_arch = "riscv64"),
     target_page_size_4k,
     target_pointer_width = "64"
 ))]
