@@ -2,26 +2,34 @@
 // Syd: rock-solid application kernel
 // src/utils/syd-elf.rs: Syd's ELF information utility
 //
-// Copyright (c) 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 //
 // SPDX-License-Identifier: GPL-3.0
 
-use std::{fs::File, os::fd::AsRawFd, process::ExitCode};
+use std::{env, fs::File, os::fd::AsRawFd, process::ExitCode};
 
 use libseccomp::{scmp_cmp, ScmpAction, ScmpFilterContext, ScmpSyscall};
-use nix::errno::Errno;
+use nix::{errno::Errno, sys::resource::Resource};
 use syd::{
-    confine::{confine_mdwe, confine_scmp_madvise, confine_scmp_wx},
+    config::{ENV_SKIP_SCMP, VDSO_SYSCALLS},
+    confine::{confine_mdwe, confine_rlimit_zero, confine_scmp_madvise, confine_scmp_wx_all},
     elf::{ElfError, ElfType, ExecutableFile, LinkingType},
     err::SydResult,
     landlock_policy::LandlockPolicy,
     path::XPathBuf,
 };
 
-// Set global allocator to mimalloc.
-#[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
+// Set global allocator to GrapheneOS allocator.
+#[cfg(all(
+    not(coverage),
+    not(feature = "prof"),
+    not(target_os = "android"),
+    not(target_arch = "riscv64"),
+    target_page_size_4k,
+    target_pointer_width = "64"
+))]
 #[global_allocator]
-static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
+static GLOBAL: hardened_malloc::HardenedMalloc = hardened_malloc::HardenedMalloc;
 
 // Set global allocator to tcmalloc if profiling is enabled.
 #[cfg(feature = "prof")]
@@ -85,21 +93,23 @@ syd::main! {
 
     let check_linking = info || is_dynamic || is_static || is_pie || is_xstack;
 
-    // Step 1: Open file.
+    // Open file.
     #[expect(clippy::disallowed_methods)]
     let file = File::open(&path)?;
 
-    // Step 2: Confine.
-    confine(&file)?;
+    // Confine unless SYD_SKIP_SCMP is set.
+    if env::var_os(ENV_SKIP_SCMP).is_none() {
+        confine(&file)?;
+    }
 
-    // Step 3: Parse ELF.
+    // Parse ELF.
     let exe = match ExecutableFile::parse(file, check_linking) {
         Ok(exe) => Some(exe),
         Err(ElfError::BadMagic) => None,
         Err(error) => return Err(error.into()),
     };
 
-    // Step 4: Report result.
+    // Report result.
     if is_script {
         return Ok(match exe {
             Some(ExecutableFile::Script) => ExitCode::SUCCESS,
@@ -167,7 +177,7 @@ syd::main! {
 
 fn help() {
     println!("Usage: syd-elf [-36dhpstxX] binary|script");
-    println!("Given a binary, file name and print ELF information.");
+    println!("Given a binary, print file name and ELF information.");
     println!("Given a script, print file name and \"SCRIPT\".");
     println!("The information line is a list of fields delimited by colons.");
     println!("Given -3, exit with success if the given binary is 32-bit.");
@@ -181,7 +191,15 @@ fn help() {
 }
 
 fn confine<Fd: AsRawFd>(fd: &Fd) -> SydResult<()> {
-    // Step 1: Set up a Landlock sandbox to disallow all access.
+    // Set nfiles, nprocs, and filesize rlimits to zero.
+    // Ignore errors as setrlimit(2) may be denied.
+    let _ = confine_rlimit_zero(&[
+        Resource::RLIMIT_FSIZE,
+        Resource::RLIMIT_NOFILE,
+        Resource::RLIMIT_NPROC,
+    ]);
+
+    // Set up a Landlock sandbox to disallow all access.
     // Ignore errors as Landlock may not be supported.
     let abi = syd::landlock::ABI::new_current();
     let policy = LandlockPolicy {
@@ -192,7 +210,7 @@ fn confine<Fd: AsRawFd>(fd: &Fd) -> SydResult<()> {
     };
     let _ = policy.restrict_self(abi);
 
-    // Step 2: Ensure W^X via MDWE (if available) and seccomp-bpf.
+    // Ensure W^X via MDWE (if available) and seccomp-bpf.
     //
     // Ignore errors as
     // 1. MDWE may be unsupported -> EINVAL.
@@ -204,9 +222,9 @@ fn confine<Fd: AsRawFd>(fd: &Fd) -> SydResult<()> {
     // Ignore errors which may mean at least one of:
     // a. CONFIG_SECCOMP_FILTER not enabled in kernel.
     // b. Syd is denying stacked seccomp cbpf filters.
-    let _ = confine_scmp_wx();
+    let _ = confine_scmp_wx_all();
 
-    // Step 3: Confine system calls to least privilege using seccomp-bpf.
+    // Confine system calls to least privilege using seccomp-bpf.
     let mut ctx = ScmpFilterContext::new(ScmpAction::KillProcess)?;
 
     // Enforce the NO_NEW_PRIVS functionality before
@@ -223,7 +241,7 @@ fn confine<Fd: AsRawFd>(fd: &Fd) -> SydResult<()> {
     let _ = ctx.set_ctl_optimize(2);
 
     // Allow base set.
-    for sysname in [
+    const BASE_SET: &[&str] = &[
         "brk",
         "exit",
         "exit_group",
@@ -236,7 +254,8 @@ fn confine<Fd: AsRawFd>(fd: &Fd) -> SydResult<()> {
         "rt_sigprocmask",
         "sigaltstack",
         "sigprocmask",
-    ] {
+    ];
+    for sysname in BASE_SET.iter().chain(VDSO_SYSCALLS) {
         ctx.add_rule(ScmpAction::Allow, ScmpSyscall::from_name(sysname)?)?;
     }
 
