@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/utils/syd-run.rs: Run a program inside a container (requires Linux-5.8 or newer).
 //
-// Copyright (c) 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 //
 // SPDX-License-Identifier: GPL-3.0
 
@@ -21,7 +21,7 @@ use syd::{
     config::SYD_SH,
     confine::{run_cmd, CLONE_NEWTIME},
     err::SydResult,
-    fs::pidfd_open,
+    fd::pidfd_open,
     path::{XPath, XPathBuf},
     proc::proc_namespaces,
 };
@@ -30,6 +30,8 @@ use syd::{
 #[cfg(all(
     not(coverage),
     not(feature = "prof"),
+    not(target_os = "android"),
+    not(target_arch = "riscv64"),
     target_page_size_4k,
     target_pointer_width = "64"
 ))]
