@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/utils/syd-tor.rs: Syd's SOCKS Proxy Forwarder
 //
-// Copyright (c) 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 // Based in part upon socksns crate which is:
 //     Copyright (c) 2020 Steven Engler
 //     SPDX-License-Identifier: MIT
@@ -40,19 +40,20 @@ use nix::{
     },
     unistd::{chdir, chroot, sysconf, write, SysconfVar},
 };
-use sendfd::RecvWithFd;
 use syd::{
     compat::{epoll_ctl_safe, set_dumpable},
-    config::{PIPE_BUF, VDSO_SYSCALLS},
+    config::{PIPE_BUF, TOR_FCNTL_OPS, TOR_PRCTL_OPS, VDSO_SYSCALLS},
     confine::{
-        confine_mdwe, confine_rlimit, confine_rlimit_zero, confine_scmp_madvise, CLONE_NEWTIME,
+        confine_mdwe, confine_rlimit, confine_rlimit_zero, confine_scmp_fcntl,
+        confine_scmp_madvise, confine_scmp_pipe2, confine_scmp_prctl, confine_scmp_sigaction,
+        confine_scmp_wx_syd, CLONE_NEWTIME,
     },
     cookie::{
         safe_connect, safe_pipe2, safe_shutdown, safe_socket, safer_accept4, CookieIdx,
         SYSCOOKIE_POOL, SYS_ACCEPT4, SYS_CONNECT, SYS_SHUTDOWN, SYS_SOCKET,
     },
     err::{SydError, SydResult},
-    fd::closeexcept,
+    fd::{closeexcept, recv_with_fd},
     hash::SydHashMap,
     ignore_signals,
     landlock_policy::LandlockPolicy,
@@ -71,6 +72,7 @@ use syd::{
     not(coverage),
     not(feature = "prof"),
     not(target_os = "android"),
+    not(target_arch = "riscv64"),
     target_page_size_4k,
     target_pointer_width = "64"
 ))]
@@ -118,7 +120,7 @@ syd::main! {
     // Receive the socket bound inside the namespace.
     let mut buf = [0u8; 1];
     let mut lfd = [-1; 1];
-    let proxy = match sock.recv_with_fd(&mut buf, &mut lfd) {
+    let proxy = match recv_with_fd(&sock, &mut buf, &mut lfd) {
         Ok((_, 1)) if lfd[0] >= 0 => {
             // Socket no longer necessary.
             drop(sock);
@@ -411,13 +413,6 @@ impl<'a> Proxy<'a> {
             "epoll_pwait",
             "epoll_pwait2",
         ];
-        let fcntl_ops: &[u64] = &[
-            libc::F_GETFD as u64,
-            libc::F_SETFD as u64,
-            libc::F_GETFL as u64,
-            libc::F_SETFL as u64,
-        ];
-        let prctl_ops: &[u64] = &[libc::PR_SET_VMA as u64];
 
         // Default allowlist.
         for name in allow_call.iter().chain(VDSO_SYSCALLS) {
@@ -430,59 +425,19 @@ impl<'a> Proxy<'a> {
         confine_scmp_madvise(&mut filter)?;
 
         // Allow safe fcntl(2) utility calls.
-        for sysname in ["fcntl", "fcntl64"] {
-            let syscall = match ScmpSyscall::from_name(sysname) {
-                Ok(syscall) => syscall,
-                Err(_) => continue,
-            };
-
-            for op in fcntl_ops {
-                filter.add_rule_conditional(
-                    ScmpAction::Allow,
-                    syscall,
-                    &[scmp_cmp!($arg1 == *op)],
-                )?;
-            }
-        }
+        confine_scmp_fcntl(&mut filter, TOR_FCNTL_OPS)?;
 
         // Allow safe prctl(2) operations.
-        let sysname = "prctl";
-        if let Ok(syscall) = ScmpSyscall::from_name(sysname) {
-            for op in prctl_ops {
-                filter.add_rule_conditional(
-                    ScmpAction::Allow,
-                    syscall,
-                    &[scmp_cmp!($arg0 == *op)],
-                )?;
-            }
-        }
+        confine_scmp_prctl(&mut filter, TOR_PRCTL_OPS)?;
 
         // Deny installing new signal handlers for {rt_,}sigaction(2).
-        for sysname in ["sigaction", "rt_sigaction"] {
-            let syscall = match ScmpSyscall::from_name(sysname) {
-                Ok(syscall) => syscall,
-                Err(_) => continue,
-            };
+        confine_scmp_sigaction(&mut filter)?;
 
-            // Installs a signal handler if first argument is non-NULL.
-            // We deny this case, but allow returning the current handler.
-            filter.add_rule_conditional(ScmpAction::Allow, syscall, &[scmp_cmp!($arg1 == 0)])?;
-        }
-
-        // Restrict pipe2(2) flags.
-        // Restrict pipe2(2) using syscall argument cookies.
-        #[expect(clippy::disallowed_methods)]
-        #[expect(clippy::useless_conversion)]
-        filter.add_rule_conditional(
-            ScmpAction::Allow,
-            ScmpSyscall::from_name("pipe2").unwrap(),
-            &[
-                scmp_cmp!($arg1 == (OFlag::O_NONBLOCK | OFlag::O_CLOEXEC).bits() as u64),
-                scmp_cmp!($arg2 == SYSCOOKIE_POOL.get(CookieIdx::Pipe2Arg2).into()),
-                scmp_cmp!($arg3 == SYSCOOKIE_POOL.get(CookieIdx::Pipe2Arg3).into()),
-                scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::Pipe2Arg4).into()),
-                scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::Pipe2Arg5).into()),
-            ],
+        // Restrict pipe2(2) flags, and use syscall argument cookies.
+        confine_scmp_pipe2(
+            &mut filter,
+            true, /*restrict_cookie*/
+            OFlag::O_CLOEXEC | OFlag::O_NONBLOCK,
         )?;
 
         // Socket filtering only works if there's no multiplexing socketcall.
@@ -582,16 +537,7 @@ impl<'a> Proxy<'a> {
         }
 
         // Prevent executable memory.
-        const PROT_EXEC: u64 = libc::PROT_EXEC as u64;
-        for name in ["mmap", "mmap2", "mprotect"] {
-            #[expect(clippy::disallowed_methods)]
-            let syscall = ScmpSyscall::from_name(name).unwrap();
-            filter.add_rule_conditional(
-                ScmpAction::Allow,
-                syscall,
-                &[scmp_cmp!($arg2 & PROT_EXEC == 0)],
-            )?;
-        }
+        confine_scmp_wx_syd(&mut filter)?;
 
         // We will ignore unshare errors next step and here we keep
         // with the expectation that we're inside the safe directory.
