@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/utils/syd-x.rs: Check executability of the given files
 //
-// Copyright (c) 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2025, 2026 Ali Polatel <alip@chesswob.org>
 //
 // SPDX-License-Identifier: GPL-3.0
 
@@ -16,7 +16,7 @@ use nix::{
 };
 use serde_json::json;
 use syd::{
-    config::HAVE_AT_EXECVE_CHECK, fs::check_executable, path::XPathBuf, proc::proc_executables,
+    config::HAVE_AT_EXECVE_CHECK, fd::check_executable, path::XPathBuf, proc::proc_executables,
 };
 
 // Set global allocator to GrapheneOS allocator.
@@ -24,6 +24,7 @@ use syd::{
     not(coverage),
     not(feature = "prof"),
     not(target_os = "android"),
+    not(target_arch = "riscv64"),
     target_page_size_4k,
     target_pointer_width = "64"
 ))]
