@@ -37,7 +37,8 @@ use crate::{
     alert,
     config::*,
     confine::{
-        confine_scmp_clone, confine_scmp_clone3, confine_scmp_fadvise, confine_scmp_madvise,
+        confine_scmp_clone, confine_scmp_clone3, confine_scmp_fadvise, confine_scmp_fcntl,
+        confine_scmp_madvise, confine_scmp_pipe2, confine_scmp_prctl, confine_scmp_sigaction,
         confine_scmp_write, scmp_add_setid_rules, ExportMode,
     },
     cookie::{safe_ftruncate64, safe_pipe2, CookieIdx, SYSCOOKIE_POOL, SYS_ACCEPT4},
@@ -197,6 +198,10 @@ impl AesWorker {
         // Use a binary tree sorted by syscall number if possible.
         let _ = ctx.set_ctl_optimize(2);
 
+        // Syscall argument cookies may be disabled
+        // at startup with trace/allow_unsafe_nocookie:1.
+        let restrict_cookie = !flags.allow_unsafe_nocookie();
+
         // SAFETY: Do NOT add supported architectures to the filter.
         // This ensures Syd can never run a non-native system call,
         // which we do not need at all.
@@ -281,7 +286,7 @@ impl AesWorker {
             Ok(syscall) => {
                 // Secure using syscall argument cookies.
                 let mut rules = vec![];
-                if !flags.allow_unsafe_nocookie() {
+                if restrict_cookie {
                     rules.extend(&[
                         scmp_cmp!($arg2 == SYSCOOKIE_POOL.get(CookieIdx::FtruncateArg2).into()),
                         scmp_cmp!($arg3 == SYSCOOKIE_POOL.get(CookieIdx::FtruncateArg3).into()),
@@ -308,7 +313,7 @@ impl AesWorker {
             Ok(syscall) => {
                 // Secure using syscall argument cookies.
                 let mut rules = vec![];
-                if !flags.allow_unsafe_nocookie() {
+                if restrict_cookie {
                     rules.extend(&[
                         #[cfg(target_arch = "x86")]
                         scmp_cmp!($arg3 == SYSCOOKIE_POOL.get(Ftruncate64Arg3).into()),
@@ -338,7 +343,7 @@ impl AesWorker {
         if let Some(syscall) = SYS_ACCEPT4.map(|n| ScmpSyscall::from_raw_syscall(n as RawSyscall)) {
             // Secure using syscall argument cookies.
             let mut rules = vec![];
-            if !flags.allow_unsafe_nocookie() {
+            if restrict_cookie {
                 rules.extend(&[
                     scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::Accept4Arg4).into()),
                     scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::Accept4Arg5).into()),
@@ -363,99 +368,25 @@ impl AesWorker {
             }
         }
 
-        // Restrict pipe2(2) flags.
-        // Restrict pipe2(2) using syscall argument cookies.
-        let sysname = "pipe2";
-        #[expect(clippy::cast_sign_loss)]
-        #[expect(clippy::useless_conversion)]
-        if let Ok(syscall) = ScmpSyscall::from_name(sysname) {
-            let mut rules = vec![scmp_cmp!($arg1 == OFlag::O_CLOEXEC.bits() as u64)];
-            if !flags.allow_unsafe_nocookie() {
-                rules.extend(&[
-                    scmp_cmp!($arg2 == SYSCOOKIE_POOL.get(CookieIdx::Pipe2Arg2).into()),
-                    scmp_cmp!($arg3 == SYSCOOKIE_POOL.get(CookieIdx::Pipe2Arg3).into()),
-                    scmp_cmp!($arg4 == SYSCOOKIE_POOL.get(CookieIdx::Pipe2Arg4).into()),
-                    scmp_cmp!($arg5 == SYSCOOKIE_POOL.get(CookieIdx::Pipe2Arg5).into()),
-                ]);
-            }
-
-            ctx.add_rule_conditional(ScmpAction::Allow, syscall, &rules)?;
-        } else {
-            info!("ctx": "confine", "op": "allow_aes_syscall",
-                "msg": format!("invalid or unsupported syscall {sysname}"));
-        }
+        // Restrict pipe2(2) flags, and use syscall argument cookies.
+        confine_scmp_pipe2(&mut ctx, restrict_cookie, OFlag::O_CLOEXEC)?;
 
         // Allow safe fcntl(2) utility calls.
-        for sysname in ["fcntl", "fcntl64"] {
-            let syscall = match ScmpSyscall::from_name(sysname) {
-                Ok(syscall) => syscall,
-                Err(_) => {
-                    info!("ctx": "confine", "op": "allow_aes_syscall",
-                        "msg": format!("invalid or unsupported syscall {sysname}"));
-                    continue;
-                }
-            };
-
-            for op in AES_FCNTL_OPS {
-                ctx.add_rule_conditional(ScmpAction::Allow, syscall, &[scmp_cmp!($arg1 == *op)])?;
-            }
-        }
+        confine_scmp_fcntl(&mut ctx, AES_FCNTL_OPS)?;
 
         // Allow safe prctl(2) operations.
-        let sysname = "prctl";
-        if let Ok(syscall) = ScmpSyscall::from_name(sysname) {
-            for (_, op) in AES_PRCTL_OPS {
-                ctx.add_rule_conditional(ScmpAction::Allow, syscall, &[scmp_cmp!($arg0 == *op)])?;
-            }
-        } else {
-            info!("ctx": "confine", "op": "allow_aes_syscall",
-                "msg": format!("invalid or unsupported syscall {sysname}"));
-        }
+        confine_scmp_prctl(&mut ctx, AES_PRCTL_OPS)?;
 
         // Deny installing new signal handlers for {rt_,}sigaction(2).
-        for sysname in ["sigaction", "rt_sigaction"] {
-            let syscall = match ScmpSyscall::from_name(sysname) {
-                Ok(syscall) => syscall,
-                Err(_) => {
-                    crate::info!("ctx": "confine", "op": "allow_main_syscall",
-                        "msg": format!("invalid or unsupported syscall {sysname}"));
-                    continue;
-                }
-            };
-
-            // Installs a signal handler if first argument is non-NULL.
-            // We deny this case, but allow returning the current handler.
-            ctx.add_rule_conditional(ScmpAction::Allow, syscall, &[scmp_cmp!($arg1 == 0)])?;
-        }
+        confine_scmp_sigaction(&mut ctx)?;
 
         // Allow safe system calls.
-        for sysname in AES_SYSCALLS.iter().chain(VDSO_SYSCALLS) {
-            match ScmpSyscall::from_name(sysname) {
-                Ok(syscall) => {
-                    ctx.add_rule(ScmpAction::Allow, syscall)?;
-                }
-                Err(_) => {
-                    info!("ctx": "confine", "op": "allow_aes_syscall",
-                        "msg": format!("invalid or unsupported syscall {sysname}"));
-                }
-            }
-        }
-
-        // Allow futex system calls.
-        for sysname in FUTEX_SYSCALLS {
-            match ScmpSyscall::from_name(sysname) {
-                Ok(syscall) => {
-                    ctx.add_rule(ScmpAction::Allow, syscall)?;
-                }
-                Err(_) => {
-                    info!("ctx": "confine", "op": "allow_aes_syscall",
-                        "msg": format!("invalid or unsupported syscall {sysname}"));
-                }
-            }
-        }
-
-        // Allow getid system calls.
-        for sysname in GET_ID_SYSCALLS {
+        for sysname in AES_SYSCALLS
+            .iter()
+            .chain(FUTEX_SYSCALLS)
+            .chain(GETID_SYSCALLS)
+            .chain(VDSO_SYSCALLS)
+        {
             match ScmpSyscall::from_name(sysname) {
                 Ok(syscall) => {
                     ctx.add_rule(ScmpAction::Allow, syscall)?;
@@ -484,6 +415,7 @@ impl AesWorker {
         Ok(ctx)
     }
 
+    #[expect(clippy::cognitive_complexity)]
     pub(crate) fn try_spawn(self) -> Result<SydJoinHandle<()>, Errno> {
         thread::Builder::new()
             .name("syd_aes".to_string())
